
/*******************************************************************************
NAME			      proj_err

PURPOSE	   	To report projection transformation package errors in the
		standard LAS manner

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
B. Ailts		9/87		Original development 
B. Ailts		12/87		Change include directory specifications
D. Steinwand		Aug, 1988	Incorporate into the LAS 5.0 projection
					transformation package.  This included
					trapping all errors generated by the
					gctp.
D. Steinwand		8/89		Added errors for Robinson & Hammer
R. Burrell			3/00		Changed to return code to caller (MODIS)

PROJECT     LAS

ALGORITHM 
    Recieve a gctp error code and print the correct error message.

ALGORITHM REFERENCES   none

ADDITIONAL COMMENTS

Errors generated internal to the gctp are listed below.  These errors have
been trapped and handled in the standard LAS manner.

  ERROR   ROUTINE     MESSAGE
  -----   ------- ---------------------------
IFLG = 0  DRIVER  SUCCESSFUL TRANSFORMATION.                      
IFLG = 1001  DRIVER  ILLEGAL INPUT SYSTEM CODE.                     
IFLG = 1002  DRIVER  ILLEGAL OUTPUT SYSTEM CODE.                   
IFLG = 1003  DRIVER  ILLEGAL INPUT UNIT CODE.                     
IFLG = 1004  DRIVER  ILLEGAL OUTPUT UNIT CODE.                   
IFLG = 1005  DRIVER  INCONSISTANT UNIT AND SYSTEM CODES FOR INPUT.        
IFLG = 1006  DRIVER  INCONSISTANT UNIT AND SYSTEM CODES FOR OUTPUT.      
IFLG = 1007  DRIVER  ILLEGAL INPUT ZONE CODE.                           
IFLG = 1008  DRIVER  ILLEGAL OUTPUT ZONE CODE.                         
IFLG = 1101  UNTFZ0  UNCOMPATABLE UNIT CODE
ERR = 1116   PAKSZ0  ILLEGAL DMS FIELD 
ERR = 001 PHI1Z0  LATITUDE FAILED TO CONVERGE AFTER',I3,' ITERATIONS
ERR = 002 PHI2Z0  LATITUDE FAILED TO CONVERGE AFTER',I3,' ITERATIONS
ERR = 003 PHI3Z0  LATITUDE FAILED TO CONVERGE AFTER',I3,' ITERATIONS
ERR = 004 PHI4Z0  LATITUDE FAILED TO CONVERGE AFTER',I3,' ITERATIONS
ERR = 011         ILLEGAL ZONE NO
ERR = 012         MISSING PROJECTION PARAMETERS
ERR = 013         MISSING PROJECTION PARAMETERS
ERR = 014         MISSING PROJECTION PARAMETERS
ERR = 011 PJ01Z0  ILLEGAL ZONE NO 
ERR = 012 PJ01Z0  MISSING PROJECTION PARAMETERS
ERR = 013 PJ01Z0  MISSING PROJECTION PARAMETERS
ERR = 014 PJ01Z0  MISSING PROJECTION PARAMETERS
ERR = 021 PJ02Z0  ILLEGAL ZONE NO 
ERR = 022 PJ02Z0  MISSING PROJECTION PARAMETERS
ERR = 023 PJ02Z0  MISSING PROJECTION PARAMETERS
ERR = 024 PJ02Z0  FAILED TO COVERGE AFTER ',I2,' ITERATIONS
ERR = 025 PJ02Z0  MISSING PROJECTION PARAMETERS
ERR = 026 PJ02Z0  FAILED TO COVERGE AFTER ',I2,' ITERATIONS
ERR = 031 PJ03Z0  EQUAL LATITUDES FOR ST. PARALLELS ON OPPOSITE SIDES OF EQUATOR
ERR = 032 PJ03Z0  MISSING PROJECTION PARAMETERS
ERR = 033 PJ03Z0  MISSING PROJECTION PARAMETERS
ERR = 034 PJ03Z0  MISSING PROJECTION PARAMETERS
ERR = 035 PJ03Z0
ERR = 041 PJ04Z0  EQUAL LATITUDES FOR ST. PARALLELS ON OPPOSITE SIDES OF EQUATOR
ERR = 042 PJ04Z0  MISSING PROJECTION PARAMETERS
ERR = 043 PJ04Z0  MISSING PROJECTION PARAMETERS
ERR = 044 PJ04Z0  POINT CANNOT BE PROJECTED
ERR = 045 PJ04Z0  MISSING PROJECTION PARAMETERS
ERR = 046 PJ04Z0
ERR = 051 PJ05Z0  MISSING PROJECTION PARAMETERS
ERR = 052 PJ05Z0  MISSING PROJECTION PARAMETERS
ERR = 053 PJ05Z0  TRANSFORMATION CANNOT BE COMPUTED AT THE POLES
ERR = 054 PJ05Z0  MISSING PROJECTION PARAMETERS
ERR = 055 PJ05Z0
ERR = 061 PJ06Z0  MISSING PROJECTION PARAMETERS
ERR = 062 PJ06Z0  MISSING PROJECTION PARAMETERS
ERR = 063 PJ06Z0  MISSING PROJECTION PARAMETERS
ERR = 064 PJ06Z0
ERR = 071 PJ07Z0  MISSING PROJECTION PARAMETERS
ERR = 072 PJ07Z0  MISSING PROJECTION PARAMETERS
ERR = 073 PJ07Z0  MISSING PROJECTION PARAMETERS
ERR = 074 PJ07Z0
ERR = 081 PJ08Z0  EQUAL LATITUDES FOR ST. PARALLELS ON OPPOSITE SIDES OF EQUATOR
ERR = 082 PJ08Z0  MISSING PROJECTION PARAMETERS
ERR = 083 PJ08Z0  MISSING PROJECTION PARAMETERS
ERR = 084 PJ08Z0  MISSING PROJECTION PARAMETERS
ERR = 085 PJ08Z0
ERR = 091 PJ09Z0  MISSING PROJECTION PARAMETERS
ERR = 092 PJ09Z0  MISSING PROJECTION PARAMETERS
ERR = 093 PJ09Z0  POINT PROJECTS INTO INFINITY
ERR = 094 PJ09Z0  MISSING PROJECTION PARAMETERS
ERR = 095 PI09Z0  LATITUDE FAILED TO CONVERGE AFTER',I3,' ITERATIONS
ERR = 101 PJ10Z0  MISSING PROJECTION PARAMETERS
ERR = 102 PJ10Z0  MISSING PROJECTION PARAMETERS
ERR = 103 PJ10Z0  POINT PROJECTS INTO INFINITY
ERR = 104 PJ10Z0  POINT PROJECTS INTO INFINITY
ERR = 111 PJ11Z0  MISSING PROJECTION PARAMETERS
ERR = 112 PJ11Z0  MISSING PROJECTION PARAMETERS
ERR = 113 PJ11Z0  POINT PROJECTS INTO A CIRCLE OF RADIUS =',F12.2, METERS
ERR = 114 PJ11Z0  MISSING PROJECTION PARAMETERS
ERR = 115 PJ11Z0  INPUT DATA ERROR
ERR = 121 PJ12Z0  MISSING PROJECTION PARAMETERS
ERR = 122 PJ12Z0  MISSING PROJECTION PARAMETERS
ERR = 123 PJ12Z0  POINT PROJECTS INTO CIRCLE OF RADIUS =',F12.2, METERS
ERR = 124 PJ12Z0  MISSING PROJECTION PARAMETERS
ERR = 125 PJ12Z0  INPUT DATA ERROR
ERR = 131 PJ13Z0  MISSING PROJECTION PARAMETERS
ERR = 132 PJ13Z0  MISSING PROJECTION PARAMETERS
ERR = 133 PJ13Z0  POINT MAPS INTO INFINITY
ERR = 134 PJ13Z0  MISSING PROJECTION PARAMETERS
ERR = 141 PJ14Z0  MISSING PROJECTION PARAMETERS
ERR = 142 PJ14Z0  MISSING PROJECTION PARAMETERS
ERR = 143 PJ14Z0  POINT CANNOT BE PROJECTED
ERR = 144 PJ14Z0  MISSING PROJECTION PARAMETERS
ERR = 145 PJ14Z0  INPUT DATA ERROR
ERR = 151 PJ15Z0  MISSING PROJECTION PARAMETERS
ERR = 152 PJ15Z0  MISSING PROJECTION PARAMETERS
ERR = 153 PJ15Z0  POINT CANNOT BE PROJECTED
ERR = 154 PJ15Z0  MISSING PROJECTION PARAMETERS
ERR = 155 PJ15Z0  INPUT DATA ERROR
ERR = 161 PJ16Z0  MISSING PROJECTION PARAMETERS
ERR = 162 PJ16Z0  MISSING PROJECTION PARAMETERS
ERR = 163 PJ16Z0  MISSING PROJECTION PARAMETERS
ERR = 164 PJ16Z0  INPUT DATA ERROR
ERR = 171 PJ17Z0  MISSING PROJECTION PARAMETERS
ERR = 172 PJ17Z0  MISSING PROJECTION PARAMETERS
ERR = 173 PJ17Z0  MISSING PROJECTION PARAMETERS
ERR = 174 PJ17Z0  INPUT DATA ERROR
ERR = 181 PJ18Z0  MISSING PROJECTION PARAMETERS
ERR = 182 PJ18Z0  MISSING PROJECTION PARAMETERS
ERR = 183 PJ18Z0  MISSING PROJECTION PARAMETERS
ERR = 191 PJ19Z0  MISSING PROJECTION PARAMETERS
ERR = 192 PJ19Z0  MISSING PROJECTION PARAMETERS
ERR = 193 PJ19Z0  MISSING PROJECTION PARAMETERS
ERR = 194 PI19Z0  INPUT DATA ERROR
ERR = 195 PI19Z0  LATITUDE FAILED TO CONVERGE AFTER',I3,' ITERATIONS
ERR = 201 PJ20Z0  INPUT DATA ERROR
ERR = 202 PJ20Z0  INPUT DATA ERROR
ERR = 203 PJ20Z0  MISSING PROJECTION PARAMETERS
ERR = 204 PJ20Z0  MISSING PROJECTION PARAMETERS
ERR = 205 PJ20Z0  POINT PROJECTS INTO INFINITY
ERR = 206 PJ20Z0  MISSING PROJECTION PARAMETERS
ERR = 211 PJ21Z0  MISSING PROJECTION PARAMETERS
ERR = 212 PJ21Z0  MISSING PROJECTION PARAMETERS
ERR = 213 PJ21Z0  MISSING PROJECTION PARAMETERS
ERR = 214 PJ21Z0  50 ITERATIONS WITHOUT CONV
ERR = 215 PJ21Z0
ERR = 221 PJ22Z0  MISSING PROJECTION PARAMETERS
ERR = 222 PJ22Z0  MISSING PROJECTION PARAMETERS
ERR = 223 PJ22Z0  MISSING PROJECTION PARAMETERS
ERR = 231 PJ23Z0  MISSING PROJECTION PARAMETERS
ERR = 232 PJ23Z0  MISSING PROJECTION PARAMETERS
ERR = 233 PJ23Z0  MISSING PROJECTION PARAMETERS
ERR = 234 PJ23Z0  TOO MANY ITERATIONS FOR INVERSE ROBINSON
ERR = 1211        Spheroid code of ',I5,' reset to 0

In LAS, these errors are handled by the proj and projon drivers and 
  will reduce to:

[proj-converge] Failed to coverge after many iterations
[proj-data] Input data error
[proj-dms] Illegal DMS field
[proj-in_conflict] Inconsistant unit and system codes for input
[proj-in_proj] Illegal input system code
[proj-in_unit] Illegal input unit code
[proj-in_zone] Illegal input zone code
[proj-infinity] Point projects into infinity
[proj-inv] Too many interations in inverse
[proj-lat_conv] Latitude failed to converge after many iterations
[proj-out_conflict] Inconsistant unit and system codes for output
[proj-out_proj] Illegal output system code
[proj-out_unit] Illegal output unit code
[proj-out_zone] Illegal output zone code
[proj-params] Missing projection parameters
[proj-poles] Transformation cannot be computed at the poles
[proj-project] Point cannot be projected
[proj-radius] Point projects into a circle of unacceptable radius
[proj-robinson] Too many iterations for inverse Robinson
[proj-som_conv] Fifty iterations performed without conversion
[proj-spheroid] Spheroid code reset to default
[proj-std_par] Equal latitudes for std parallels on opposite sides of equator
[proj-zone] Illegal zone no

******************************************************************************/
#include "shared_resample.h"

int proj_err( long err )
{
    int error_code = MRT_NO_ERROR;

    switch ( err )
    {
	case -2:
	    error_code = ERROR_GCTP_PTBREAK;
	    break;
	case 0:
	    error_code = MRT_NO_ERROR;
	    break;
	case 6:
	    error_code = ERROR_GCTP_OUTNAME;
	    break;
	case 46:
	case 35:
	case 55:
	case 64:
	case 74:
	case 85:
	case 215:
	    error_code = ERROR_GCTP_TRANSFAIL;
	    break;
	case 24:
	case 26:
	case 251:
	case 241:
	    error_code = ERROR_GCTP_CONVERGE;
	    break;
	case 234:
	    error_code = ERROR_GCTP_ROBITER;
	    break;
	case 235:
	case 236:
	    error_code = ERROR_GCTP_INVITER;
	    break;
	case 125:
	case 115:
	case 145:
	case 155:
	case 164:
	case 174:
	case 194:
	case 201:
	case 202:
	case 252:
	    error_code = ERROR_GCTP_INDATA;
	    break;
	case 1116:
	    error_code = ERROR_GCTP_BADDMS;
	    break;
	case 1005:
	    error_code = ERROR_GCTP_INUNIT;
	    break;
	case 1001:
	    error_code = ERROR_GCTP_INSYS;
	    break;
	case 1101:
	case 1003:
	    error_code = ERROR_GCTP_INCODE;
	    break;
	case 1007:
	    error_code = ERROR_GCTP_INZONE;
	    break;
	case 93:
	case 103:
	case 104:
	case 205:
	case 133:
	    error_code = ERROR_GCTP_INF;
	    break;
	case 95:
	case 1:
	case 2:
	case 3:
	case 4:
	case 195:
	    error_code = ERROR_GCTP_LATITER;
	    break;
	case 1006:
	    error_code = ERROR_GCTP_OUTUNIT;
	    break;
	case 1002:
	    error_code = ERROR_GCTP_OUTSYS;
	    break;
	case 1004:
	    error_code = ERROR_GCTP_OUTCODE;
	    break;
	case 1008:
	    error_code = ERROR_GCTP_OUTZONE;
	    break;
	case 53:
	    error_code = ERROR_GCTP_POLAR;
	    break;
	case 44:
	case 143:
	case 153:
	    error_code = ERROR_GCTP_PTPROJ;
	    break;
	case 113:
	case 123:
	    error_code = ERROR_GCTP_PTCIRC;
	    break;
	case 214:
	    error_code = ERROR_GCTP_50ITER;
	    break;
	case 1211:
	    error_code = ERROR_GCTP_SPHERE;
	    break;
	case 81:
	case 31:
	case 41:
	    error_code = ERROR_GCTP_EQLAT;
	    break;
	case 11:
	case 21:
	    error_code = ERROR_GCTP_BADZONE;
	    break;
	case 22:
	    error_code = ERROR_GCTP_SPPAR;
	    break;
	case 5:
	    error_code = ERROR_GCTP_BADCODE;
	    break;
	case 94:
	case 25:
	case 12:
	case 13:
	case 14:
	case 23:
	case 32:
	case 33:
	case 34:
	case 42:
	case 43:
	case 45:
	case 51:
	case 52:
	case 54:
	case 61:
	case 62:
	case 63:
	case 71:
	case 72:
	case 73:
	case 82:
	case 83:
	case 84:
	case 91:
	case 92:
	case 101:
	case 102:
	case 111:
	case 112:
	case 144:
	case 114:
	case 121:
	case 122:
	case 124:
	case 131:
	case 132:
	case 134:
	case 141:
	case 142:
	case 151:
	case 152:
	case 154:
	case 161:
	case 162:
	case 163:
	case 181:
	case 182:
	case 183:
	case 191:
	case 192:
	case 193:
	case 206:
	case 211:
	case 212:
	case 213:
	case 203:
	case 204:
	case 171:
	case 172:
	case 173:
	case 221:
	case 222:
	case 223:
	case 231:
	case 232:
	case 233:
	    error_code = ERROR_GCTP_MISPAR;
	    break;
    }
    return ( error_code );
}
